@@ -120,41 +120,27 @@ class HRRRtoGSSHA(GRIDtoGSSHA):
     """This class converts the HRRR output data to GSSHA formatted input.
     This class inheris from class:`GRIDtoGSSHA`.
 
-    Attributes:
-        gssha_project_folder(:obj:`str`): Path to the GSSHA project folder
-        gssha_project_file_name(:obj:`str`): Name of the GSSHA elevation grid file.
-        lsm_input_folder_path(:obj:`str`): Path to the input folder for the LSM files.
-        lsm_search_card(:obj:`str`): Glob search pattern for LSM files. Ex. "*.grib2".
-        lsm_lat_var(Optional[:obj:`str`]): Name of the latitude variable in the LSM netCDF files. Defaults to 'lat'.
-        lsm_lon_var(Optional[:obj:`str`]): Name of the longitude variable in the LSM netCDF files. Defaults to 'lon'.
-        lsm_time_var(Optional[:obj:`str`]): Name of the time variable in the LSM netCDF files. Defaults to 'time'.
-        lsm_lat_dim(Optional[:obj:`str`]): Name of the latitude dimension in the LSM netCDF files. Defaults to 'lat'.
-        lsm_lon_dim(Optional[:obj:`str`]): Name of the longitude dimension in the LSM netCDF files. Defaults to 'lon'.
-        lsm_time_dim(Optional[:obj:`str`]): Name of the time dimension in the LSM netCDF files. Defaults to 'time'.
-        output_timezone(Optional[:obj:`tzinfo`]): This is the timezone to output the dates for the data. Default is the timezone of your GSSHA model. This option does NOT currently work for NetCDF output.
-
     Example::
 
         from gsshapy.grid import HRRRtoGSSHA
 
-        l2g = HRRRtoGSSHA(gssha_project_folder='E:\\GSSHA',
+        h2g = HRRRtoGSSHA(gssha_project_folder='E:\\GSSHA',
                           gssha_project_file_name='gssha.prj',
                           lsm_input_folder_path='E:\\GSSHA\\hrrr-data',
-                          lsm_search_card="*.grib2",
-                          )
+                          lsm_search_card="*.grib2")
 
         # example data var map
         data_var_map_array = [
-                               ['precipitation_rate', 'PRATE_P0_L1_GLC0'],
-                               ['pressure', 'PRES_P0_L1_GLC0'],
-                               ['relative_humidity', 'RH_P0_L103_GLC0'],
-                               ['wind_speed', ['UGRD_P0_L103_GLC0', 'VGRD_P0_L103_GLC0']],
-                               ['direct_radiation_cc', ['DSWRF_P0_L1_GLC0', 'TCDC_P0_L10_GLC0']],
-                               ['diffusive_radiation_cc', ['DSWRF_P0_L1_GLC0', 'TCDC_P0_L10_GLC0']],
-                               ['temperature', 'TMP_P0_L1_GLC0'],
-                               ['cloud_cover_pc', 'TCDC_P0_L10_GLC0'],
-                              ]
-
+            ['precipitation_rate', 'PRATE_P0_L1_GLC0'],
+            ['pressure', 'PRES_P0_L1_GLC0'],
+            ['relative_humidity', 'RH_P0_L103_GLC0'],
+            ['wind_speed', ['UGRD_P0_L103_GLC0', 'VGRD_P0_L103_GLC0']],
+            ['direct_radiation_cc', ['DSWRF_P0_L1_GLC0', 'TCDC_P0_L10_GLC0']],
+            ['diffusive_radiation_cc', ['DSWRF_P0_L1_GLC0',
+                                        'TCDC_P0_L10_GLC0']],
+            ['temperature', 'TMP_P0_L1_GLC0'],
+            ['cloud_cover_pc', 'TCDC_P0_L10_GLC0'],
+        ]
     """
     def __init__(self,
                  gssha_project_folder,
@@ -170,7 +156,40 @@ class HRRRtoGSSHA(GRIDtoGSSHA):
                  output_timezone=None,
                  ):
         """
-        Initializer function for the HRRRtoGSSHA class
+        Initializer function for the HRRRtoGSSHA class.
+
+        Parameters
+        ----------
+        gssha_project_folder: :obj:`str`
+            Path to the GSSHA project folder.
+        gssha_project_file_name: :obj:`str`
+            Name of the GSSHA elevation grid file.
+        lsm_input_folder_path: :obj:`str`
+            Path to the input folder for the LSM files.
+        lsm_search_card: :obj:`str`
+            Glob search pattern for LSM files. Ex. "*.grib2".
+        lsm_lat_var: :obj:`str`, optional
+            Name of the latitude variable in the LSM netCDF files.
+            Defaults to 'lat'.
+        lsm_lon_var: :obj:`str`, optional
+            Name of the longitude variable in the LSM netCDF files.
+            Defaults to 'lon'.
+        lsm_time_var: :obj:`str`, optional
+            Name of the time variable in the LSM netCDF files.
+            Defaults to 'time'.
+        lsm_lat_dim: :obj:`str`, optional
+            Name of the latitude dimension in the LSM netCDF files.
+            Defaults to 'lat'.
+        lsm_lon_dim: :obj:`str`, optional
+            Name of the longitude dimension in the LSM netCDF files.
+            Defaults to 'lon'.
+        lsm_time_dim: :obj:`str`, optional
+            Name of the time dimension in the LSM netCDF files.
+            Defaults to 'time'.
+        output_timezone: :obj:`tzinfo`, optional
+            This is the timezone to output the dates for the data.
+            Default is the timezone of your GSSHA model.
+            This option does NOT currently work for NetCDF output.
         """
         super(HRRRtoGSSHA, self).__init__(gssha_project_folder,
                                           gssha_project_file_name,
